#!/hxmt/soft/Develop/anaconda2/bin/python
# -*- coding: utf-8 -*-
"""
coordinate convert from J2000 to Satellite or Satellite to J2000

example: 
        >>>import coordtrans
        >>>[theta,phi] = coordtrans.J2000_Sat(ra=[10.0],dec=[20.0],start_time='2017-06-20T01:00:00')
        >>>[ra,dec] = coordtrans.Sat_J2000(theta=[10.0],phi=[20.0],start_time='2017-06-20T01:00:00')

Created on Wed Sep 26 00:45:12 2018

@author: hy
modified by zy
 line170: def SearchFile():
    fdir = os.path.abspath(os.path.dirname(__file__))+'/'
    with open(fdir+'config','r') as f:

"""

import glob
import numpy as np
from scipy import interpolate
from astropy import units as u
from .Quaternion import * 
from astropy.io import fits
from astropy.time import Time
from astropy.coordinates.representation import (CartesianRepresentation, UnitSphericalRepresentation)
import sys
import os

def J2000_Sat(ra=[0.0],dec=[0.0],start_time='2018-09-01T00:20:00',end_time=None,step=1,filename=None):
    # ra,dec in degree. list or array
    # time format: yyyy-mm-ddTHH:MM:SS
    
    if end_time is None:
        end_time = start_time
    
    # search Att file
    if not filename:
        filename = SearchFile(start_time,end_time)
    print('Att Filename: ')
    print(filename)  
    
    [q1,q2,q3,time] = ReadAttfile(filename)
    
    # interp, nearest
    [q1s,q2s,q3s] = Interq(start_time,end_time,step,q1,q2,q3,time)
    print(q1s.shape)
        
    q0s = np.sqrt(np.ones(q1s.shape,dtype=q1s.dtype) - q1s ** 2 - q2s ** 2 - q3s ** 2)
    quat = np.vstack((q1s,q2s,q3s,q0s))
    theta = np.empty(q1s.shape,dtype=q1s.dtype)
    phi = np.empty(q1s.shape,dtype=q1s.dtype)
    
    ra = np.array(ra)
    dec = np.array(dec)
    if ra.ndim == 1 and dec.ndim == 1:
        
        if ra.shape == dec.shape:
            if ra.shape[0] == 1:
                ra = np.zeros(q1s.shape,dtype=q1s.dtype) + ra
                dec = np.zeros(q1s.shape,dtype=q1s.dtype) + dec
            elif ra.shape == q1s.shape:
                ra = ra
                dec = dec
            else:
                print('RA/Dec and time must have the same size!')
                sys.exit()
        else:
            print('RA and Dec must have the same size!')
            sys.exit()
                
            
    else:
        print('RA/Dec must be a 1-Dimensional array or list!')
#        raise SystemExit
        sys.exit()
    
    #q_xyz2zxy = Qt.Quat(([[0,0,1],[1,0,0],[0,1,0]]))
    q_xyz2zxy = Quat(([[0,0,1],[1,0,0],[0,1,0]])) #X.song
    for index, quat1 in enumerate(quat.T):
        #quat1 = Qt.Quat(quat1)
        quat1 = Quat(quat1) #X.song
#        quat2 = q_xyz2zxy * quat1
        print(ra)
        quat1= quat1 * q_xyz2zxy
        sph1 = UnitSphericalRepresentation(ra[index]*u.deg,dec[index]*u.deg)
        cart1 = sph1.represent_as(CartesianRepresentation)
        cart1_xyz = cart1.xyz
        cart2_xyz = np.matmul(quat1.transform.T,cart1_xyz)
        cart2 = CartesianRepresentation(cart2_xyz[0],cart2_xyz[1],cart2_xyz[2])
        sph2 = cart2.represent_as(UnitSphericalRepresentation)   
        
        phi[index] = sph2.lon.deg
        theta[index] = 90 - sph2.lat.deg
        
        print('RA Dec: ', ra[index], dec[index])
        print('theta phi: ',theta[index],phi[index])
  
    return theta, phi


def Sat_J2000(theta=[0.0],phi=[0.0],start_time='2017-06-16T00:00:00',end_time=None,step=1,filename=None):
    # theta,phi in degree. list or array
    # time format: yyyy-mm-ddTHH:MM:SS
    
    if end_time is None:
        end_time = start_time
    
    # search Att file
    if not filename:
        filename = SearchFile(start_time,end_time)
    print('Att Filename: ')
    print(filename)  
    
    [q1,q2,q3,time] = ReadAttfile(filename)
    
    # interp, nearest
    [q1s,q2s,q3s] = Interq(start_time,end_time,step,q1,q2,q3,time)
        
    q0s = np.sqrt(np.ones(q1s.shape,dtype=q1s.dtype) - q1s ** 2 - q2s ** 2 - q3s ** 2)
    quat = np.vstack((q1s,q2s,q3s,q0s))
    ra = np.empty(q1s.shape,dtype=q1s.dtype)
    dec = np.empty(q1s.shape,dtype=q1s.dtype)
    
    theta = np.array(theta)
    phi = np.array(phi)
    if theta.ndim == 1 and phi.ndim == 1:
        
        if theta.shape == phi.shape:
            if theta.shape[0] == 1:
                theta = np.zeros(q1s.shape,dtype=q1s.dtype) + theta
                phi = np.zeros(q1s.shape,dtype=q1s.dtype) + phi
            elif theta.shape == q1s.shape:
                theta = theta
                phi = phi
            else:
                print('THETA/PHI and time must have the same size!')
                sys.exit()
        else:
            print('THETA and PHI must have the same size!')
            sys.exit()
                
            
    else:
        print('THETA/PHI must be a 1-Dimensional array or list!')
#        raise SystemExit
        sys.exit()
    
    #q_xyz2zxy = Qt.Quat(([[0,0,1],[1,0,0],[0,1,0]]))
    q_xyz2zxy = Quat(([[0,0,1],[1,0,0],[0,1,0]])) #X.song
    for index, quat1 in enumerate(quat.T):
        #quat1 = Qt.Quat(quat1)
        quat1 = Quat(quat1) #X.song
#        quat1 = q_xyz2zxy * quat1
        quat1= quat1 * q_xyz2zxy
        sph1 = UnitSphericalRepresentation(phi[index]*u.deg,(90.0-theta[index])*u.deg)
        cart1 = sph1.represent_as(CartesianRepresentation)
        cart1_xyz = cart1.xyz
        cart2_xyz = np.matmul(quat1.transform,cart1_xyz)
        cart2 = CartesianRepresentation(cart2_xyz[0],cart2_xyz[1],cart2_xyz[2])
        sph2 = cart2.represent_as(UnitSphericalRepresentation)   
        
        ra[index] = sph2.lon.deg
        dec[index] = sph2.lat.deg
        
        print('RA Dec: ', ra[index], dec[index])
        print('theta phi: ',theta[index],phi[index])
  
    return theta, phi



def SearchFile(start_time,end_time):
    fdir = os.path.abspath(os.path.dirname(__file__))+'/'
    with open(fdir+'config','r') as f:
        tmp = f.readline()
    rootpath = tmp.split( )[2] #path/to/1K/file/
    
    utctime_start = Time(start_time,scale='utc',format='isot')
    utctime_end = Time(end_time,scale='utc',format='isot')
    dt = (utctime_end - utctime_start) * 24 #hour
    att_file = []

    for i in range(0,int(dt.value)+1):
        utctime2 = Time(utctime_start.mjd+i/24.,scale='utc',format='mjd')
        utctimestr = utctime2.isot

        search_path = rootpath + 'Y' + utctimestr[0:4] + utctimestr[5:7] \
        + '/' + utctimestr[0:4] + utctimestr[5:7] + utctimestr[8:10] + '-*' \
        + '/HXMT_' + utctimestr[0:4] + utctimestr[5:7] + utctimestr[8:13]
        #path/to/1K/file/Y201801/20180101/...
        print('Search path: ', search_path)
        tmp = glob.glob(search_path + '_Att_*.FITS')
        if not tmp:
            print('File not exist! UTC Time: ', utctimestr[0:13])
            sys.exit()
        else:
            if(tmp not in att_file):
                att_file.append(tmp)
     
    return att_file



def ReadAttfile(filename):
    # read file
    time = np.array([])
    q1 = np.array([])
    q2 = np.array([])
    q3 = np.array([])
    for tmpname in filename:
        print('Read file: ', tmpname[0])
        #print('Read file: ', tmpname)
        hdu = fits.open(tmpname[0])
        #hdu = fits.open(tmpname)
        time = np.hstack((time,hdu[3].data.field('Time')))
        q1 = np.hstack((q1,hdu[3].data.field('Q1')))
        q2 = np.hstack((q2,hdu[3].data.field('Q2')))
        q3 = np.hstack((q3,hdu[3].data.field('Q3')))
        hdu.close()# Dr.X.Song added
    
    # sort
    index = np.argsort(time)
    time = time[index]
    q1 = q1[index]
    q2 = q2[index]
    q3 = q3[index]
    
    return q1,q2,q3,time

def Interq(start_time,end_time,step,q1,q2,q3,time):
    # interp data
    start_time = Time(start_time,scale='utc',format='isot')
    start_times = (start_time.mjd-55927)*86400. + 3
    end_time = Time(end_time,scale='utc',format='isot')
    end_times = (end_time.mjd-55927)*86400. + 3
    
    times = np.arange(start_times,end_times+step,step)
    f1 = interpolate.interp1d(time,q1,kind='nearest')
    q1s = f1(times)
    f2 = interpolate.interp1d(time,q2,kind='nearest')
    q2s = f2(times)
    f3 = interpolate.interp1d(time,q3,kind='nearest')
    q3s = f3(times)
    
    return q1s,q2s,q3s


if __name__=="__main__":
    x1 = float(sys.argv[1])
    x2 = float(sys.argv[2])
    startutc = sys.argv[3]
    ctype = int(sys.argv[4])
    if ctype == 1:
        [theta,phi] = J2000_Sat(ra=[x1],dec=[x2],start_time=startutc)
    elif ctype == 2:
        [ra,dec] = Sat_J2000(theta=[x1],phi=[x2],start_time=startutc)
    else:
        print('Wrong Type! 1 for J2000 to Satellite; 2 for Satellite to J2000')
