#   -*- coding: utf-8 -*-   #
# author Mr. Xue Wangchen   #
# for deadtime calculation  #
#############################

 
import numpy as np
from astropy.io import fits
import time

def TotDeadTime(filename, tstart, tend, binwidth):
    ''' ``interval`` should be a list containing 2 nums. This function will return total dead time list during given interval in unit μs. '''
    hdu = fits.open(filename)
    deadList = [] # length: tbins elements length: idetec
    # for timeinterval bin
    ntbins= int((tend -tstart)/binwidth)
    if ntbins<1:
        ntbins=1
    for itime in range(ntbins):
        interval= [tstart + binwidth*itime, tstart +binwidth*(itime+1)]
        deadtimePerbin_low=[]
        deadtimePerbin_hig=[]
        # for each detector
        for idetec in hdu[3:]:
            time = idetec.data.field('TIME')
            dead = idetec.data.field('DEAD_TIME')
            gain_type = idetec.data.field('GAIN_TYPE')
            lowgain_index = np.where(gain_type == 1)[0]
            higgain_index = np.where(gain_type == 0)[0]

            startindex=np.where(time>=interval[0])[0]
            endindex= np.where(time<interval[1])[0]

            timeintervalIndex= list(set(startindex.tolist()) & set(endindex.tolist()))
            timeintervalIndex_low= list(set(timeintervalIndex) & set(lowgain_index))
            timeintervalIndex_hig= list(set(timeintervalIndex) & set(higgain_index))

            dead_low=dead[np.array(timeintervalIndex_low)]
            dead_hig=dead[np.array(timeintervalIndex_hig)]
            deadtimePerbin_low.append(np.sum(dead_low) * 0.000000001)#*0.000001 tentatively, because of large luminosity of MC, we don't use micro in real data
            deadtimePerbin_hig.append(np.sum(dead_hig) * 0.000000001)

        deadtimePerbin_low.extend(deadtimePerbin_hig)
        deadList.append(deadtimePerbin_low)
    return deadList

'''
if __name__=='__main__' :
    beginTime=time.time()
    file = '/home/songxy/tn201201_055336/gag_evt_tn201201_055336_fa_v00.fits'
    tstart = 60501167.1414179 +50
    tend= tstart +10
    tbinwidth=1.0
    print(TotDeadTime(file, tstart, tend, tbinwidth))
    print(time.time() - beginTime)
'''