import os
import sys

from PyQt5.QtCore import *
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *

import matplotlib
from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.backends.backend_qt5 import NavigationToolbar2QT as NavigationToolbar
from matplotlib.figure import Figure
import matplotlib.animation as animation

from astropy.io import fits
import numpy as np  
import threading


sys.path.append(os.path.abspath(os.path.dirname(__file__)+'/'+'..'))
from data import *
from tools import *
from fitEdepos import *
from fitBG import *
from .GUI_mainframe_GECAM import *

######################################################################################  
class MPLbase(FigureCanvas):
      def __init__(self):
          self.fig = Figure()
          #self.axes = self.fig.add_axes([0.1,0.1,0.8,0.8])
          self.axes = self.fig.add_axes()
          FigureCanvas.__init__(self, self.fig)
          FigureCanvas.setSizePolicy(self, QSizePolicy.Expanding, QSizePolicy.Expanding)
          FigureCanvas.updateGeometry(self)
#        self.Figure = matplotlib.figure.Figure(figsize=(7,6))
#        self.axes = self.Figure.add_axes([0.1,0.1,0.8,0.8])
#        self.FigureCanvas = FigureCanvas(self,-1,self.Figure)
      #    self.NavigationToolbar = NavigationToolbar(self)
          self.mpl_connect2=self.fig.canvas.mpl_connect('button_press_event', self.onclick)

      def subplot(self, *args,**kwargs):
          return self.fig.add_subplot(*args,**kwargs)
#      def setfigtitle(self, titlestring='title'):
#          self.fig.title(titlestring)
          

      def UpdatePlot(self):
          '''''#修改图形的任何属性后都必须使用self.UpdatePlot()更新GUI界面 '''
          self.draw()
  
  
      def plot(self,*args,**kwargs):
          '''''#最常用的绘图命令plot '''
          self.axes.plot(*args,**kwargs)
          self.UpdatePlot()
  
      def errorbar(self,*args,**kwargs):
          '''''#最常用的绘图命令plot '''
          self.axes = self.fig.add_axes([0.1,0.1,0.8,0.8])
          self.axes.errorbar(*args,**kwargs)
          self.UpdatePlot()
  
  
      def semilogx(self,*args,**kwargs):
          ''''' #对数坐标绘图命令 '''
          self.axes.semilogx(*args,**kwargs)
          self.UpdatePlot()
  
      def semilogy(self,*args,**kwargs):
          ''''' #对数坐标绘图命令 '''
          self.axes.semilogy(*args,**kwargs)
          self.UpdatePlot()
  
      def loglog(self,*args,**kwargs):
          ''''' #对数坐标绘图命令 '''
          self.axes.loglog(*args,**kwargs)
          self.UpdatePlot()


      def grid(self,flag=True):
          ''''' ##显示网格  '''
          if flag:
              self.axes.grid()
          else:
              self.axes.grid(False)


      def title_MPL(self,TitleString="GECAM in python by X.Song"):
          ''''' # 给图像添加一个标题   '''
          self.axes.set_title(TitleString)
  
  
      def xlabel(self,LabelString="X"):
          ''''' # Add xlabel to the plotting    '''
          self.axes.set_xlabel(LabelString)
  
  
      def ylabel(self,LabelString="Y"):
          ''''' # Add ylabel to the plotting '''
          self.axes.set_ylabel(LabelString)
  
  
      def xticker(self,major_ticker=1.0,minor_ticker=0.1):
          ''''' # 设置X轴的刻度大小 '''
          self.axes.xaxis.set_major_locator( MultipleLocator(major_ticker) )
          self.axes.xaxis.set_minor_locator( MultipleLocator(minor_ticker) )
      def yticker(self,major_ticker=1.0,minor_ticker=0.1):
          ''''' # 设置Y轴的刻度大小 '''
  
  
      def legend(self,*args,**kwargs):
          ''''' #图例legend for the plotting  '''
          self.axes.legend(*args,**kwargs)


      def xlim(self,x_min,x_max):
          ''''' # 设置x轴的显示范围  '''
          self.axes.set_xlim(x_min,x_max)
  
  
      def ylim(self,y_min,y_max):
          self.axes.set_ylim(y_min,y_max)
  
      def getylim(self):
          return self.axes.get_ylim()  
        


      def savefig(self,*args,**kwargs):
          self.fig.savefig(*args,**kwargs)


      def cla(self):
          self.axes = self.fig.add_axes([0.1,0.1,0.8,0.8])
          #self.axes = self.fig.add_axes()
          self.axes.clear()
          self.fig.set_canvas(self)
          self.UpdatePlot()

      def fixylim(self):
          tup=self.getylim()
          self.ylim(tup[0],tup[1])



      def onclick(self,event):
          detID=glv.detID_currentselected
          if glv.selectBGrange[detID] ==1 and len(glv.corx_BG[detID])<4:
             if len(glv.corx_BG[detID])==0:
                glv.corx_BG[detID]=[event.xdata]
             else:
                glv.corx_BG[detID].extend([event.xdata])
             #glv.cory_BG[detID].append(event.ydata)
             self.buildline()
          elif glv.selectROIrange[detID] ==1 and len(glv.corx_ROI[detID])<2:
               #devide into two satelite A ,B
               if detID<50:
                  for idetec in range(50):
                      if len(glv.corx_ROI[idetec])==0: 
                          glv.corx_ROI[idetec]=[event.xdata]
                      else:
                          glv.corx_ROI[idetec]=[glv.corx_ROI[idetec][0],event.xdata] 
               else:
                  for idetec in range(50):
                      if len(glv.corx_ROI[idetec+50])==0:
                          glv.corx_ROI[idetec+50]=[event.xdata]
                      else:
                          glv.corx_ROI[idetec+50]=[glv.corx_ROI[idetec+50][0],event.xdata]
               self.buildline()
          #elif glv.flag_currentinmainframe=='lightcurve' and glv.selectsource_lc[detID] ==1 and len(glv.corx_selectsource_lc[detID])<2:
          elif glv.flag_currentinmainframe=='lightcurve' and glv.selectsource_lc[detID] ==1:
               if len(glv.corx_selectsource_lc[detID])==1:
                  glv.corx_selectsource_lc[detID].append(event.xdata)
                  self.buildline()
               elif len(glv.corx_selectsource_lc[detID])==2 or len(glv.corx_selectsource_lc[detID])==0:
                  glv.corx_selectsource_lc[detID]=[event.xdata]   
                  self.buildline()
          #elif glv.flag_currentinmainframe=='Edepos' and glv.selectsource_Edepos[detID] ==1 and len(glv.corx_selectsource_Edepos[detID])<2:
          elif glv.flag_currentinmainframe=='Edepos' and glv.selectsource_Edepos[detID] ==1:
               print('1111111111'+str(detID)+str(glv.corx_selectsource_Edepos[detID] ))
               if len(glv.corx_selectsource_Edepos[detID])==0 or len(glv.corx_selectsource_Edepos[detID])==2:
                  glv.corx_selectsource_Edepos[detID]=[event.xdata]
                  self.buildline()
                  print('22222222222'+str(detID)+str(glv.corx_selectsource_Edepos[detID])) 
               elif len(glv.corx_selectsource_Edepos[detID])==1:
                  glv.corx_selectsource_Edepos[detID].append(event.xdata)
                  self.buildline()
                  print('33333333333'+str(detID)+str(glv.corx_selectsource_Edepos[detID]))
               #glv.cory_selectsource_Edepos[detID].append(event.ydata)
          elif glv.flag_currentinmainframe=='Edepos' and glv.selectsource_fitEdepos[detID] ==1:
               if len(glv.corx_selectsource_fitEdepos[detID])==0:
                  glv.corx_selectsource_fitEdepos[detID]=[event.xdata]
                  self.buildline()
               elif len(glv.corx_selectsource_fitEdepos[detID])==1:
                  glv.corx_selectsource_fitEdepos[detID].append(event.xdata)
                  self.buildline()
                  #glv.selectsource_fitEdepos[detID]=0
               else:
                  pass
          else:
              pass 

      def buildline(self):  
           detID=glv.detID_currentselected
           if glv.selectBGrange[detID] ==1 and glv.selectROIrange[detID] ==0:
              #print('1111')
              lenx=len(glv.corx_BG[detID])
              #leny=len(glv.cory_BG[detID])               
              tup=self.getylim() 
              if lenx==0:
                print('pls click and select position!')
              else:
                x=glv.corx_BG[detID][lenx-1]
                self.fixylim()
                self.plot([x,x],[tup[0],tup[1]])  
                self.ylim(tup[0],tup[1])

           elif glv.flag_currentinmainframe=='lightcurve' and glv.selectROIrange[detID] ==1:
              #print('2222')
              lenx=len(glv.corx_ROI[detID])
              #leny=len(glv.cory_ROI[detID])               
              tup=self.getylim() 
              if lenx==0:
                print('pls click and select position!')
              else:
                x=glv.corx_ROI[detID][lenx-1]
                self.fixylim()
                self.plot([x,x],[tup[0],tup[1]])  
                self.ylim(tup[0],tup[1])

           elif  glv.flag_currentinmainframe=='lightcurve' and glv.selectsource_lc[detID] ==1:
                #print('3333')
                lenx=len(glv.corx_selectsource_lc[detID])
                tup=self.getylim() 
                if lenx==0:
                  print('pls click and select position!')
                else:
                  x=glv.corx_selectsource_lc[detID][lenx-1]
                  self.fixylim()
                  self.plot([x,x],[tup[0],tup[1]])  
                  self.ylim(tup[0],tup[1])
           elif  glv.flag_currentinmainframe=='Edepos' and glv.selectsource_Edepos[detID] ==1:
                 #print('should print here, on bld line')
                 lenx=len(glv.corx_selectsource_Edepos[detID])
                 #print(glv.corx_selectsource_Edepos[detID]) 
                 tup=self.getylim() 
                 if lenx==0:
                   print('pls click and select position!') 
                 else:
                   x=glv.corx_selectsource_Edepos[detID][lenx-1]
                   self.fixylim()
                   self.plot([x,x],[tup[0],tup[1]])  
                   self.ylim(tup[0],tup[1])
           elif  glv.flag_currentinmainframe=='Edepos' and glv.selectsource_fitEdepos[detID] ==1:
                 #print('should print here, on bld line')
                 lenx=len(glv.corx_selectsource_fitEdepos[detID])
                 #print(glv.corx_selectsource_Edepos[detID]) 
                 tup=self.getylim()
                 if lenx==0:
                   print('pls click and select position!')
                 else:
                   x=glv.corx_selectsource_fitEdepos[detID][lenx-1]
                   self.fixylim()
                   self.plot([x,x],[tup[0],tup[1]])
                   self.ylim(tup[0],tup[1])

           else:
               pass 


#
#         #try:
#         #  x=glv.corx[lenx-1]
#         #  tup=self.axes.getylim()
      def fill_between(self,*args,**kwargs):
          self.axes.fill_between(*args,**kwargs)  
          self.UpdatePlot()
         #  self.plot([x,x],[tup[0],tup[1]])  
         #  #y=glv.cory[leny-1]
         #  #self.plt.axvline(x)
         #  #self.plt.show()
         #except Exception:
         #  print('pls click and select position!')        
         #else:
         #  print('one boundary is selected!')


class MPLwrapper(QWidget):
      def __init__(self , parent =None):
        QWidget.__init__(self, parent)
        self.canvas = MPLbase()
        self.vbl = QVBoxLayout()
        self.ntb = NavigationToolbar(self.canvas, parent)
        self.vbl.addWidget(self.ntb)
        self.vbl.addWidget(self.canvas)
        self.setLayout(self.vbl)

      def plotwitherr(self,*args,**kwargs): 
          self.canvas.errorbar(*args,**kwargs)
      def plot(self, *args, **kwargs):
          self.canvas.plot(*args, **kwargs)
      def fixylim(self):
          self.canvas.fixylim()
      def clearplot(self):
          self.canvas.cla()
      def setxylog(self,*args,**kwargs):
          self.canvas.loglog(*args,**kwargs)

      def setxlog(self,*args,**kwargs):
          self.canvas.semilogx(*args,**kwargs)

      def setylog(self,*args,**kwargs):
          self.canvas.semilogy(*args,**kwargs)

      def fill_between(self, *args, **kwargs):
          self.canvas.fill_between(*args, **kwargs)
      def xlabel(self, LabelString='X'):
          self.canvas.xlabel(LabelString)
      def ylabel(self, LabelString='Y'):
          self.canvas.ylabel(LabelString)
      def title_MPL(self,TitleString="GECAM in python by X.Song"):
          self.canvas.title_MPL(TitleString)
      def legend(self,*args,**kwargs):
          self.canvas.legend(*args,**kwargs)
#      def title_fig(self, TitleString="GECAM in python by X.Song"):
#          self.canvas.setfigtitle(TitleString)
      def UpdatePlot(self):
          self.canvas.UpdatePlot()
      
      def getylim(self):
          return self.canvas.getylim()
      def ylim(self,y_min,y_max):
          self.canvas.ylim(y_min,y_max)
      def savefig(self,*args,**kwargs):
          self.canvas.savefig(*args,**kwargs)

