#!/hxmt/soft/Develop/anaconda2/bin/python
# -*- coding: utf-8 -*-
"""
coordinate convert from J2000 to Satellite or Satellite to J2000

example: 
        >>>import coordtrans
        >>>[theta,phi] = coordtrans.J2000_Sat(ra=[10.0],dec=[20.0],start_time='2017-06-20T01:00:00')
        >>>[ra,dec] = coordtrans.Sat_J2000(theta=[10.0],phi=[20.0],start_time='2017-06-20T01:00:00')

Created on Wed Sep 26 00:45:12 2018

@author: hy
modified by zy
 line170: def SearchFile():
    fdir = os.path.abspath(os.path.dirname(__file__))+'/'
    with open(fdir+'config','r') as f:

modified 2019-11-26 by hy:  def ReadAttfile(filename): q1 = np.hstack((q1,hdu[1].data.field('QUATERNION')[:,0]))
   def J2000_Sat: q_xyz2zxy = Quat(([[1,0,0],[0,-1,0],[0,0,-1]])) 
   
modified 2020-01-06 by hy: Using scipi.spatial.transform
        
"""

import glob
import numpy as np
from scipy import interpolate
from astropy.io import fits
from astropy.time import Time
import sys
import os
from scipy.spatial.transform import Rotation

def J2000_Sat(ra=[0.0],dec=[0.0],start_time='2018-09-01T00:20:00',end_time=None,step=1,filename=None):
    # ra,dec in degree. list or array
    # time format: yyyy-mm-ddTHH:MM:SS
    
    if end_time is None:
        end_time = start_time
    
    # search Att file
    if not filename:
        filename = SearchFile(start_time,end_time)
    print('Att Filename: ')
    print(filename)  
    
    [q0,q1,q2,q3,time] = ReadAttfile(filename)
    
    # interp, nearest
    [q0s,q1s,q2s,q3s,timelist] = Interq(start_time,end_time,step,q0,q1,q2,q3,time)
    print(q1s.shape)
        
    quat = np.vstack((q1s,q2s,q3s,q0s))
    
    # norm 
    quat = Rotation.from_quat(quat.T,normalized=False)
    
    ra = np.array(ra)
    dec = np.array(dec)
    print(ra,type(ra))
    if ra.ndim == 1 and dec.ndim == 1:
        
        if ra.shape == dec.shape:
            if ra.shape[0] == 1:
                ra = np.zeros(q1s.shape,dtype=q1s.dtype) + ra
                dec = np.zeros(q1s.shape,dtype=q1s.dtype) + dec
            elif ra.shape == q1s.shape:
                ra = ra
                dec = dec
            else:
                print('RA/Dec and time must have the same size!')
                sys.exit()
        else:
            print('RA and Dec must have the same size!')
            sys.exit()
                
            
    else:
        print('RA/Dec must be a 1-Dimensional array or list!')
#        raise SystemExit
        sys.exit()
        
    dcm1 = quat
    #print('Quat: ',dcm1.as_quat())
    #print('Dcm1: ', dcm1.as_dcm())
    dcm2 = Rotation.from_dcm([[1,0,0],[0,-1,0],[0,0,-1]])
    dcm = dcm1 * dcm2
    [x,y,z] = sph2cart(np.radians(ra),np.radians(dec))
    #print('X:',x, ' Y:',y, ' Z:',z)
    vec_J2000 = np.array([x,y,z])
    vec_sat = dcm.inv().apply(vec_J2000.T)
    phi,theta = cart2sph(vec_sat[:,0], vec_sat[:,1], vec_sat[:,2])
    theta = 90 - theta*180/np.pi
    phi = phi*180/np.pi
    
    return theta, phi, timelist


def Sat_J2000(theta=[0.0],phi=[0.0],start_time='2017-06-16T00:00:00',end_time=None,step=1,filename=None):
    # theta,phi in degree. list or array
    # time format: yyyy-mm-ddTHH:MM:SS
    
    if end_time is None:
        end_time = start_time
    
    # search Att file
    if not filename:
        filename = SearchFile(start_time,end_time)
    print('Att Filename: ')
    print(filename)  
    
    [q0,q1,q2,q3,time] = ReadAttfile(filename)
    
    # interp, nearest
    [q0s,q1s,q2s,q3s,timelist] = Interq(start_time,end_time,step,q0,q1,q2,q3,time)
        
    quat = np.vstack((q1s,q2s,q3s,q0s))
    # norm 
    quat = Rotation.from_quat(quat.T,normalized=False)

    theta = np.array(theta)
    phi = np.array(phi)
    if theta.ndim == 1 and phi.ndim == 1:
        
        if theta.shape == phi.shape:
            if theta.shape[0] == 1:
                theta = np.zeros(q1s.shape,dtype=q1s.dtype) + theta
                phi = np.zeros(q1s.shape,dtype=q1s.dtype) + phi
            elif theta.shape == q1s.shape:
                theta = theta
                phi = phi
            else:
                print('THETA/PHI and time must have the same size!')
                sys.exit()
        else:
            print('THETA and PHI must have the same size!')
            sys.exit()
                
            
    else:
        print('THETA/PHI must be a 1-Dimensional array or list!')
#        raise SystemExit
        sys.exit()
    
    dcm1 = quat
    dcm2 = Rotation.from_dcm([[1,0,0],[0,-1,0],[0,0,-1]])
    dcm = dcm2.inv() * dcm1.inv()
    [x,y,z] = sph2cart(phi*np.pi/180.,(90.-theta)*np.pi/180.)
    vec_sat = np.array([x,y,z])
    vec_J2000 = dcm.inv().apply(vec_sat.T)

    phi,theta = cart2sph(vec_J2000[:,0], vec_J2000[:,1], vec_J2000[:,2])
    dec = theta*180/np.pi
    ra = phi*180/np.pi

    #-------------------corrected by ql----------------
    try:
        if len(ra)>1:
            ra_maintain_dex = np.greater(ra,0)
            ra_inverse_dex = (1-ra_maintain_dex.astype(int)).astype(bool)
            ra[ra_inverse_dex] = 360 + ra[ra_inverse_dex]
    except:
        if ra<0:
            ra = 360 + ra
  
    return ra,dec

def cart2sph(x,y,z):
    hxy = np.hypot(x,y)
    dec = np.arctan2(z,hxy)
    ra = np.arctan2(y,x)
    return ra,dec

def sph2cart(ra,dec):
    costheta = np.cos(dec)
    x = costheta * np.cos(ra)
    y = costheta * np.sin(ra)
    z = np.sin(dec)
    return x,y,z

def SearchFile(start_time,end_time):
    fdir = os.path.abspath(os.path.dirname(__file__))+'/'
    with open(fdir+'config','r') as f:
        tmp = f.readline()
    rootpath = tmp.split( )[2] #path/to/1K/file/
    
    utctime_start = Time(start_time,scale='utc',format='isot')
    utctime_end = Time(end_time,scale='utc',format='isot')
    dt = (utctime_end - utctime_start) * 24 #hour
    att_file = []

    for i in range(0,int(dt.value)+1):
        utctime2 = Time(utctime_start.mjd+i/24.,scale='utc',format='mjd')
        utctimestr = utctime2.isot

        search_path = rootpath + 'Y' + utctimestr[0:4] + utctimestr[5:7] \
        + '/' + utctimestr[0:4] + utctimestr[5:7] + utctimestr[8:10] + '-*' \
        + '/HXMT_' + utctimestr[0:4] + utctimestr[5:7] + utctimestr[8:13]
        #path/to/1K/file/Y201801/20180101/...
        print('Search path: ', search_path)
        tmp = glob.glob(search_path + '_Att_*.FITS')
        if not tmp:
            print('File not exist! UTC Time: ', utctimestr[0:13])
            sys.exit()
        else:
            if(tmp not in att_file):
                att_file.append(tmp)
     
    return att_file



def ReadAttfile(filename):
    # read file
    time = np.array([])
    q1 = np.array([])
    q2 = np.array([])
    q3 = np.array([])
    q0 = np.array([])
    for tmpname in filename:
        #print(tmpname)
        #print('Read file: ', tmpname[0])
        hdu = fits.open(tmpname)
        #time = np.hstack((time,hdu[1].data.field('TIME')))
        #q1 = np.hstack((q1,hdu[1].data.field('QUATERNION')[:,0]))
        #q2 = np.hstack((q2,hdu[1].data.field('QUATERNION')[:,1]))
        #q3 = np.hstack((q3,hdu[1].data.field('QUATERNION')[:,2]))
        #q0 = np.hstack((q0,hdu[1].data.field('QUATERNION')[:,3]))
	    #----------------------------new QUATERNION version-------------------------
	    #QUATERNION = np.vstack((file[1].data.field('Q1'),file[1].data.field('Q2'),file[1].data.field('Q3'),file[1].data.field('Q4'))).T
        time = np.hstack((time, hdu[1].data.field('TIME')))
        q1 = np.hstack((q1,hdu[1].data.field('Q1')))
        q2 = np.hstack((q2,hdu[1].data.field('Q2')))
        q3 = np.hstack((q3,hdu[1].data.field('Q3')))
        q0 = np.hstack((q0,hdu[1].data.field('Q4')))

    #print(q1,q1[0])
    # sort
    index = np.argsort(time)
    time = time[index]
    q1 = q1[index]
    q2 = q2[index]
    q3 = q3[index]
    q0 = q0[index]
       
    return q0,q1,q2,q3,time

def Interq(start_time,end_time,step,q0,q1,q2,q3,time):
    # interp data
    start_time = Time(start_time,scale='utc',format='isot')
    start_times = (start_time.mjd-58484)*86400.
    #print('MET:', start_times)
    end_time = Time(end_time,scale='utc',format='isot')
    end_times = (end_time.mjd-58484)*86400
    

    #-------------------corrected by ql----------------
    try:
        if len(start_times)>1:
            times = np.zeros((start_times.shape[0],int(end_times[0]+step-start_times[0])))
            #if len(start_time)>1:
            for i in range(start_times.shape[0]):
                times[i,:] = np.arange(start_times[i],end_times[i]+step,step)
    except:
        times = np.arange(start_times,end_times+step,step)

    #-------------------before-------------------------
    #times = np.arange(start_times,end_times+step,step)

    timelist = Time((times - 0)/86400. + 58484,scale='utc',format='mjd')
    f0 = interpolate.interp1d(time,q0,kind='nearest')
    q0s = f0(times)    
    f1 = interpolate.interp1d(time,q1,kind='nearest')
    q1s = f1(times)
    f2 = interpolate.interp1d(time,q2,kind='nearest')
    q2s = f2(times)
    f3 = interpolate.interp1d(time,q3,kind='nearest')
    q3s = f3(times)


    #-------------------corrected by ql----------------
    try:
        if timelist.shape[0]>1:
            timelist_tem = []
            q0s_tem = []
            q1s_tem = []
            q2s_tem = []
            q3s_tem = []
            for i in range(timelist.shape[0]):
                timelist_tem.append(timelist[i].value[0])
                q0s_tem.append(q0s[i,0])
                q1s_tem.append(q1s[i,0])
                q2s_tem.append(q2s[i,0])
                q3s_tem.append(q3s[i,0])
            q0s = np.array(q0s_tem)
            q1s = np.array(q1s_tem)
            q2s = np.array(q2s_tem)
            q3s = np.array(q3s_tem)
            timelist = timelist_tem
    except:
        pass
    return q0s,q1s,q2s,q3s,timelist


if __name__=="__main__":
    x1 = float(sys.argv[1])
    x2 = float(sys.argv[2])
    startutc = sys.argv[3]
    ctype = int(sys.argv[4])
    if ctype == 1:
        [theta,phi] = J2000_Sat(ra=[x1],dec=[x2],start_time=startutc)
    elif ctype == 2:
        [ra,dec] = Sat_J2000(theta=[x1],phi=[x2],start_time=startutc)
    else:
        print('Wrong Type! 1 for J2000 to Satellite; 2 for Satellite to J2000')
