"""
Created on Sat Dec 21 15:13:47 2019

@author: Dr. Xinying Song 
"""
from astropy.io import fits
import numpy as np
import math
import time
import pathlib
import os
import sys
sys.path.append(os.path.abspath(os.path.dirname(__file__)+'/'+'..'))
from tools import * 
from .globalvar_GECAM import *

def gethist_t(tnet, timeinterval, tbinw): #with given tbinw, return histogram for t, note that: here the t is a evt data

    ini_bins=int(timeinterval/tbinw)
    hist=np.histogram(tnet, ini_bins)
    atime=np.delete(hist[1],ini_bins,axis=0)+tbinw/2.0
    counts_perT=hist[0]/tbinw
    counts_perT_err=np.array([math.sqrt(x) for x in hist[0]])/tbinw
    return atime,counts_perT, counts_perT_err
def gethist_countrate_perT(at, tbinw):#with given tbinw, return histogram for t, note that: here the t is a hist already
    counts_perT=at/tbinw
    counts_perT_err=at**0.5/tbinw
    return counts_perT, counts_perT_err


def gethist_e(edepos, t_interval, detID): #with given ebinw, return hist for e, note that: here the e is a evt data
    Edepos= glv.lEbin_mid[detID]
    hist=np.histogram(edepos, glv.ebin_array)
    counts_perE=hist[0]/glv.lEbin_width[detID]/t_interval
    counts_perE_err=np.array([math.sqrt(x) for x in hist[0]])/glv.lEbin_width[detID]/t_interval
    return np.array(Edepos), counts_perE, counts_perE_err

def gethist_countrate_perE(aedepos, timeinterval,detID):#with given ebinw, return hist for e, note that: here the e is already hist
    Edepos=[]
    if glv.selectsource_Edepos[detID]==1:
       Eindex=getindex_GECAM.getindex_le(glv.corx_selectsource_Edepos[detID],detID)
       Edepos= glv.lEbin_mid[detID][Eindex[0]:Eindex[1]+1]
       Ebinwidth= glv.lEbin_width[detID][Eindex[0]:Eindex[1]+1]
    else:
       Edepos= glv.lEbin_mid[detID]
       Ebinwidth= glv.lEbin_width[detID]

    counts_perE=aedepos/Ebinwidth/timeinterval
    counts_perE_err=aedepos**0.5/Ebinwidth/timeinterval

    return np.array(Edepos), counts_perE, counts_perE_err

def gethist_t(tnet, timeinterval, tbinw): #with given tbinw, return histogram for t, note that: here the t is a evt data

    ini_bins=int(timeinterval/tbinw)
    hist=np.histogram(tnet, ini_bins)
    atime=np.delete(hist[1],ini_bins,axis=0)+tbinw/2.0
    counts_perT=hist[0]/tbinw
    counts_perT_err=np.array([math.sqrt(x) for x in hist[0]])/tbinw
    return atime,counts_perT, counts_perT_err

def gethist_countrate_perT(at, tbinw):#with given tbinw, return histogram for t, note that: here the t is a hist already
    counts_perT=at/tbinw
    counts_perT_err=at**0.5/tbinw
    return counts_perT, counts_perT_err


def gethist_e(edepos, t_interval, detID): #with given ebinw, return hist for e, note that: here the e is a evt data
    Edepos= glv.lEbin_mid[detID]
    hist=np.histogram(edepos, glv.ebin_array)
    counts_perE=hist[0]/glv.lEbin_width[detID]/t_interval
    counts_perE_err=np.array([math.sqrt(x) for x in hist[0]])/glv.lEbin_width[detID]/t_interval
    return np.array(Edepos), counts_perE, counts_perE_err

def gethist_countrate_perE(aedepos, timeinterval,detID):#with given ebinw, return hist for e, note that: here the e is already hist
    Edepos=[]
    if glv.selectsource_Edepos[detID]==1:
       Eindex=getindex_GECAM.getindex_le(glv.corx_selectsource_Edepos[detID],detID)
       Edepos= glv.lEbin_mid[detID][Eindex[0]:Eindex[1]+1]
       Ebinwidth= glv.lEbin_width[detID][Eindex[0]:Eindex[1]+1]
    else:
       Edepos= glv.lEbin_mid[detID]
       Ebinwidth= glv.lEbin_width[detID]

    counts_perE=aedepos/Ebinwidth/timeinterval
    counts_perE_err=aedepos**0.5/Ebinwidth/timeinterval
    return np.array(Edepos), counts_perE, counts_perE_err
'''descriminate high/low gain'''
def discriminate_HLgain( fitsrec_tcg ):
    te=np.vstack((fitsrec_tcg.field(0),fitsrec_tcg.field(1)))
    gain_type=fitsrec_tcg.field(2)
    lowgain_index=np.where(gain_type==1)# old:1 new: 1   
    higgain_index=np.where(gain_type==0)# old:2 new: 0   
    te=te.T
    ate_low=te[lowgain_index[0]]
    ate_hig=te[higgain_index[0]]
    return ate_low.T, ate_hig.T
    
    
'''
def sorted_highlowgain( fitsrec_tcg ):
'''
def recogSatLabel(filenamelist):
    ''' for A/B joint trig, burst data '''
    if(len(filenamelist) == 2):
      return 'j'
    elif(len(filenamelist) == 1):
      if 'gag' in filenamelist[0]:
         return 'a'
      else:
         return 'b' 
    else:
      pass
def getData(filenamelist, Ndetec_tot, tbinwidth):
    ''' initialize the lte, time, Echannel vs counts '''
    lte=[]
    ltime=[]

    lcounts_perT=[]
    lcounts_perT_err=[]

    date_obs=''
    date_end=''
    date_ref=''
    trig_ori=''
    datatype=''


    lstarttime=[]
    lendtime=[]
    ltrigtime=[]


    ebins=0
    Emid=[]
    Ebinwidth=[]
    ebound=np.array([0])

   
    location_J2000onSat=[]


    beginTime= time.time()
    for trigfile in filenamelist:
        lte_low=[]                            
        lte_hig=[]
        ltime_low=[]
        ltime_hig=[]
        lcounts_perT_low=[]                
        lcounts_perT_err_low=[]
        lcounts_perT_hig=[]
        lcounts_perT_err_hig=[]
        with fits.open(trigfile) as hdu:
             ''' assign time start/stop bins by defalut'''
             date_obs=hdu[0].header['DATE_OBS']
             date_end=hdu[0].header['DATE_END']
             date_ref=hdu[0].header['DATE_REF']
             trig_ori=hdu[0].header['LOC_ORI']
             datatype=hdu[0].header['DATATYPE']
             starttime= float(hdu[0].header['TSTART'])
             endtime  = float(hdu[0].header['TSTOP'])
             trigtime = float(hdu[0].header['BST_TIME'])
             location_J2000onSat.append([hdu[0].header['RA_OBJ'], hdu[0].header['DEC_OBJ']])
             timeinterval = endtime - starttime
             tbins    = int(timeinterval/tbinwidth)


             '''for EC relationship'''
             ECbounds = hdu[1].data
             channel = ECbounds.field(0)
             ebins=channel.shape[0]
             E_ledges = ECbounds.field(1)
             E_redges = ECbounds.field(2)
             Ebin_mid= 0.5*(E_ledges + E_redges)
             Ebin_width = E_redges - E_ledges
             ebound = E_ledges.tolist()
             ebound.append(E_redges[-1])
             ebound = np.array(ebound)

             '''Emid, Ebinwidth for h/l gain'''
             Emid.extend([Ebin_mid]*Ndetec_tot*2) 
             Ebinwidth.extend([Ebin_width]*Ndetec_tot*2) 
         
             ''' for every detector assignment for t and e '''
             for idetec in range(Ndetec_tot):
                 fitsrec_timechannelgain=hdu[idetec + 3].data # np.array((t,e,))
                 ''' sorted by gain type '''
                 #array_te_lowgain, array_te_higgain =sorted_highlowgain(fitsrec_timechannelgain )                  
                 array_te_lowgain, array_te_higgain =discriminate_HLgain(fitsrec_timechannelgain )
                 '''assignment for lcounts in low/high gain'''
                 ''' low gain '''
                 tnet= array_te_lowgain[0] -starttime 
                 array_te_lowgain[0] = tnet
                 ini_binwidth= tbinwidth #in default
                 '''time histograme '''
                 atime, tcount, tcount_err=gethist_t(tnet, timeinterval, ini_binwidth)
                 ltime_low.append(atime.tolist())
                 lcounts_perT_low.append(tcount.tolist())
                 lcounts_perT_err_low.append(tcount_err.tolist()) 

                 ''' high gain '''
                 tnet= array_te_higgain[0]-starttime 
                 array_te_higgain[0]=tnet
                 ini_binwidth= tbinwidth #in default
                 '''time histograme '''
                 atime, tcount, tcount_err=gethist_t(tnet, timeinterval, ini_binwidth)
                 ltime_hig.append(atime.tolist())
                 lcounts_perT_hig.append(tcount.tolist())
                 lcounts_perT_err_hig.append(tcount_err.tolist())


                 '''gen 2D array for time and channel '''
                 lte_low.append(array_te_lowgain)
                 lte_hig.append(array_te_higgain)
             #for one satellte, detector loop ends
             ltime.extend(ltime_low)
             ltime.extend(ltime_hig)
             lcounts_perT.extend(lcounts_perT_low) 
             lcounts_perT.extend(lcounts_perT_hig) 
             lcounts_perT_err.extend(lcounts_perT_err_low) 
             lcounts_perT_err.extend(lcounts_perT_err_hig)
             lte.extend(lte_low)
             lte.extend(lte_hig)

             lstarttime.append(starttime)
             lendtime.append(endtime)
             ltrigtime.append(trigtime)
    return date_obs,date_end,date_ref, trig_ori,datatype,lstarttime, lendtime,ltrigtime[0],location_J2000onSat[0],\
           ltime, lcounts_perT, lcounts_perT_err,\
           lte, \
           ebins, Emid, Ebinwidth, ebound
def setValue_trigdata(filenamelist):

    satlabel=recogSatLabel(filenamelist)
    ''' ini'''
    Ndetec_tot = glv.Ndetec_tot  
    tbinwidth = glv.tbinwidth
    if 'bn' in filenamelist[0]:
        index_b=filenamelist[0].index('bn')
    else:
        index_b=filenamelist[0].index('tn')
    
    taskID= filenamelist[0][index_b+2: index_b+15]

    ''' extract the data file dir '''
    if 'gag' in glv.lfilename[0]:
       gagindex=glv.lfilename[0].index('gag')
    elif 'gbg' in glv.lfilename[0]:
       gagindex=glv.lfilename[0].index('gbg')
    else:
       self.signal_filebrowse_to_status_box.emit('the filename is anormal!')

    genfitsDir= filenamelist[0][0:gagindex] + 'fitsforXspec/'
    rspoutputdir= genfitsDir
    ''' extract data from trig file'''
    date_obs, date_end, date_ref, trig_ori, datatype, lstarttime, lendtime,trigtimeMET, locationJ2000_onSat,\
     ltime, lcounts_perT, lcounts_perT_err,\
     lte,\
     ebins, Emid, Ebinwidth, ebound = getData(filenamelist, Ndetec_tot, tbinwidth)
    ltimeinterval=(np.array(lendtime) - np.array(lstarttime)).tolist()
    ''' assignment for global value class '''
    '''if sat b and single , complete the lte, ... by zero'''
    if satlabel =='b':
        satAblankArray = [0]
        satAblankArray.extend(ltimeinterval)
        ltimeinterval=satAblankArray

        satAblankArray=Ndetec_tot*2*[0]
        satAblankArray.extend(lte)
        lte=satAblankArray

        satAblankArray = Ndetec_tot * 2 * [0]
        satAblankArray.extend(ltime)
        ltime=satAblankArray

        satAblankArray = Ndetec_tot * 2 * [0]
        satAblankArray.extend(lcounts_perT)
        lcounts_perT=satAblankArray

        satAblankArray = Ndetec_tot * 2 * [0]
        satAblankArray.extend(lcounts_perT_err)
        lcounts_perT_err=satAblankArray

        satAblankArray = Ndetec_tot * 2 * [0]
        satAblankArray.extend(Ebinwidth)
        Ebinwidth=satAblankArray

        satAblankArray = Ndetec_tot * 2 * [0]
        satAblankArray.extend(Emid)
        Emid=satAblankArray

    glv.satLabel        = satlabel
    glv.lfilename       = filenamelist
    glv.lte             = lte
    glv.lstarttime       = lstarttime 
    glv.lendtime         = lendtime
    glv.trigtimeMET      = trigtimeMET
    glv.trigtimeUTC      = TTtoUTC.met2utc(trigtimeMET)
    glv.ltimeinterval    = ltimeinterval
    glv.ltbins           = [int(x) for x in ((np.array(lendtime)- np.array(lstarttime))/tbinwidth).tolist()] 
    glv.ltime           = ltime
    glv.lcounts_perT    = lcounts_perT
    glv.lcounts_perT_err= lcounts_perT_err
    glv.ebins           = ebins  
    glv.ebin_edges      = np.arange(-0.5,ebins + 0.5,1.0) #  
    glv.lEbin_width     = Ebinwidth
    glv.lEbin_mid       = Emid
    glv.ebound= ebound
    glv.taskID=taskID
    glv.genfitsDir=genfitsDir
    glv.rspoutputdir=rspoutputdir
    glv.locationJ2000_onSat= locationJ2000_onSat


          
