import os
import sys
sys.path.append(os.path.abspath(os.path.dirname(__file__)+'/'+'..'))

import numpy as np
from scipy.optimize import curve_fit
import matplotlib.pyplot as plt
import math

import time
from tools import gen1Dhist_GECAM, proerr_GECAM, getindex_GECAM, toggle_GECAM, chisq_ndof_GECAM, dtcor_GECAM
from .polymodel_GECAM import  *


class genProjection(object):
      def __init__(self):
          self.detID_selected=[]
          self.lte=[]
          self.corx_ROI=[]
          self.corx_BG=[]
          self.tbins=100 
          self.tbinwidth=1.0
          self.ebins=20 
          self.ebin_edges=[] 
          self.bgpoly=[] 
          self.bgpcov=[] 
          self.bgpolyorder=1
          self.deadtime=0.0
          

      def getindex_lt(self, lx,tbinwidth):
          return getindex_GECAM.getindex_lt(lx,tbinwidth)
      def getindex_le(self, *args,**kwargs):
          return getindex_GECAM.getindex_le( *args,**kwargs)
      def shape2Dhist(self, *args,**kwargs):
          return toggle_GECAM.shape2Dhist(*args,**kwargs)
      def proerr(self, *args,**kwargs):
          return proerr_GECAM.proerr( *args,**kwargs)

      def setpara(self, detID_selected,
                  lte,
                  corx_ROI,
                  corx_BG,
                  tbins, tbinwidth,ebins, ebin_edges,
                  bgpoly,
                  bgpcov,
                  bgpolyorder,
                  deadtime):
          self.detID_selected=detID_selected        
          self.lte=lte        
          self.corx_ROI=corx_ROI
          self.corx_BG=corx_BG
          self.tbins=tbins
          self.tbinwidth=tbinwidth
          self.ebins= ebins
          self.ebin_edges=ebin_edges
          self.bgpoly=bgpoly
          self.bgpcov=bgpcov
          self.bgpolyorder=bgpolyorder
          self.deadtime=deadtime


      def getbg_ROI(self,lte_detID, ebins, corx_ROI, corx_BG,
                    bgpoly, bgpov,bgpolyorder,tbins,tbinwidth,ebin_edges):
          het2D, fulltime = self.shape2Dhist(lte_detID, tbins, tbinwidth, ebin_edges)
          let2D=het2D.tolist()
          BGcount_ROI=[]
          BGcounterr_ROI=[]
          indexBG=getindex_GECAM.getindex_lt(corx_BG,tbinwidth)
          for iebins in range(ebins):
              popt= bgpoly[iebins]
              poly = np.poly1d(popt)
              pcov= bgpov[iebins]
              polyint = np.polyint(poly)
              order=bgpolyorder
              bgnumreal=np.sum(let2D[iebins][indexBG[0]:indexBG[1]+1])\
                        + np.sum(let2D[iebins][indexBG[2]:indexBG[3]+1])
              integROI= polyint(corx_ROI[1])-polyint(corx_ROI[0])
              integBG = polyint(corx_BG[1]+0.5*tbinwidth)-polyint(corx_BG[0]-0.5*tbinwidth)+\
                        polyint(corx_BG[3]+0.5*tbinwidth)-polyint(corx_BG[2]-0.5*tbinwidth)
              numerr = (self.proerr(corx_ROI, order, pcov)) ** 0.5
              coeff=bgnumreal/ integBG
              if integROI == 0 or integBG ==0:
                  BGcounterr_ROI.append(0)
                  BGcount_ROI.append(0)
              else:
                  BGcounterr_ROI.append(numerr * coeff)
                  BGcount_ROI.append(coeff*integROI)
          return np.array(BGcount_ROI), np.array(BGcounterr_ROI)




      '''
      # calculate the BG projection of ROI

      def getbg_ROI(self, lte_detID, corx_ROI,
                    tbins, tbinwidth,ebins, ebin_edges,
                    bgpoly, bgpcov, bgpolyorder):
          lindex_ROI= self.getindex_lt(corx_ROI,tbinwidth)
          BGcount_ROI=[]
          BGcounterr_ROI=[]

          het2D, fulltime=self.shape2Dhist(lte_detID,tbins, tbinwidth, ebin_edges)
          st=fulltime[lindex_ROI[0]:lindex_ROI[1]+1]
          for i in range(ebins):
              popt=bgpoly[i]
              pcov=bgpcov[i]
              order=bgpolyorder
              poly=np.poly1d(popt)
              polyint=np.polyint(poly)
              num=[poly(x) for x in st]
              #for count & err
              BGcount_ROI.append(sum(num))
              #for err
              numerr=(self.proerr(corx_ROI, order, pcov))**0.5
              numint=polyint(corx_ROI[1])-polyint(corx_ROI[0])
              if numint ==0:
                 BGcounterr_ROI.append(0)
              else:
                 BGcounterr_ROI.append(numerr*sum(num)/numint)
          BGcount_ROI=np.array(BGcount_ROI)
          BGcounterr_ROI=np.array(BGcounterr_ROI)
          return  BGcount_ROI, BGcounterr_ROI
      '''

      def genEdepos_projection(self):
          '''
          init
          '''
          lte=self.lte
          detID_selected=self.detID_selected
          corx_ROI= self.corx_ROI
          corx_BG=self.corx_BG
          tbins= self.tbins
          tbinwidth= self.tbinwidth
          ebins= self.ebins
          ebin_edges= self.ebin_edges
          bgpoly= self.bgpoly
          bgpcov= self.bgpcov
          bgpolyorder= self.bgpolyorder
          deadtime=self.deadtime


          BGEcounts       = 50*[np.array([0])]
          BGEcounts_err   = 50*[np.array([0])]
          BGtime_interval = 50*[0.0]
          netEcounts      = 50*[np.array([0])]
          netEcounts_err  = 50*[np.array([0])]
          nettime_interval =50*[0.0]
          fullEcounts     = 50*[np.array([0])]
          fullEcounts_err = 50*[np.array([0])]
          fulltime_interval=50*[0.0]
          print('-------------------------------------------------')
          print(len(bgpoly))
          print(len(detID_selected))
          for idetec in detID_selected:
              idx=detID_selected.index(idetec)
              BGcount_ROI, BGcounterr_ROI =self.getbg_ROI(lte[idetec],ebins,
                                                          corx_ROI, corx_BG,
                                                          bgpoly[idx], bgpcov[idx],
                                                          bgpolyorder[idx],tbins, tbinwidth,
                                                           ebin_edges)
              '''
              BGcount_ROI, BGcounterr_ROI= self.getbg_ROI(lte[idetec], corx_ROI,
                                                          tbins, tbinwidth,
                                                          ebins, ebin_edges,
                                                          bgpoly[idx], bgpcov[idx], bgpolyorder[idx])
              '''
              '''
              if TR or very short burst, the corx_ROI is very narrow, even equal or narrower than the BG tbinwdith,
              In the toggle_t2e, the index get :getindex_lt(lt,tbinwidth) will return 
              incorrect results, so tbinwidth will be narrower than the input value   
              '''
              tbinwidth_forNarrowROI= tbinwidth/40.
              het2D, fulltime=self.shape2Dhist(lte[idetec],tbins*40, tbinwidth_forNarrowROI, ebin_edges)
              timeinterval_ROI=corx_ROI[1]- corx_ROI[0]
              slice_et2D, edepos, at =toggle_GECAM.toggle_t2e(het2D, corx_ROI,tbinwidth_forNarrowROI)
              netEcounts[idx]= edepos- BGcount_ROI
              netEcounts_err[idx]= (edepos + BGcounterr_ROI**2)**0.5
              nettime_interval[idx]=dtcor_GECAM.deadtime_cor(np.sum(netEcounts[idx]), timeinterval_ROI, deadtime)
      
              fullEcounts[idx]=edepos
              fullEcounts_err[idx]=edepos**0.5
              fulltime_interval[idx]=dtcor_GECAM.deadtime_cor(np.sum(fullEcounts[idx]),timeinterval_ROI, deadtime)
      
              BGEcounts[idx]=BGcount_ROI
              BGEcounts_err[idx]=BGcounterr_ROI
              BGtime_interval[idx]=dtcor_GECAM.deadtime_cor(np.sum(BGEcounts[idx]),timeinterval_ROI, deadtime)
      
          return netEcounts, netEcounts_err, nettime_interval, fullEcounts,\
                 fullEcounts_err, fulltime_interval, BGEcounts,BGEcounts_err,\
                 BGtime_interval

      def genEdepos_projection_single(self):
          '''
          init
          '''
          lte = self.lte
          detID_selected = self.detID_selected
          corx_ROI = self.corx_ROI
          corx_BG = self.corx_BG
          tbins = self.tbins
          tbinwidth = self.tbinwidth
          ebins = self.ebins
          ebin_edges = self.ebin_edges
          bgpoly = self.bgpoly
          bgpcov = self.bgpcov
          bgpolyorder = self.bgpolyorder
          deadtime = self.deadtime

          BGEcounts = np.array([0])
          BGEcounts_err = np.array([0])
          BGtime_interval = 0.0
          netEcounts = np.array([0])
          netEcounts_err = np.array([0])
          nettime_interval = 0.0
          fullEcounts = np.array([0])
          fullEcounts_err = np.array([0])
          fulltime_interval = 0.0

          for idetec in detID_selected:
              idx = idetec
              BGcount_ROI, BGcounterr_ROI = self.getbg_ROI(lte[idetec], ebins,
                                                           corx_ROI, corx_BG,
                                                           bgpoly[idetec], bgpcov[idetec],
                                                           bgpolyorder[idetec], tbins, tbinwidth,
                                                           ebin_edges)
              '''
              BGcount_ROI, BGcounterr_ROI= self.getbg_ROI(lte[idetec], corx_ROI,
                                                          tbins, tbinwidth,
                                                          ebins, ebin_edges,
                                                          bgpoly[idx], bgpcov[idx], bgpolyorder[idx])
              '''
              '''
              if TR or very short burst, the corx_ROI is very narrow, even equal or narrower than the BG tbinwdith,
              In the toggle_t2e, the index get :getindex_lt(lt,tbinwidth) will return 
              incorrect results, so tbinwidth will be narrower than the input value   
              '''
              tbinwidth_forNarrowROI = tbinwidth / 40.
              het2D, fulltime = self.shape2Dhist(lte[idetec], tbins * 40, tbinwidth_forNarrowROI, ebin_edges)
              timeinterval_ROI = corx_ROI[1] - corx_ROI[0]
              slice_et2D, edepos, at = toggle_GECAM.toggle_t2e(het2D, corx_ROI, tbinwidth_forNarrowROI)
              netEcounts = edepos - BGcount_ROI
              netEcounts_err = (edepos + BGcounterr_ROI ** 2) ** 0.5
              #nettime_interval = dtcor_GECAM.deadtime_cor(np.sum(netEcounts[idx]), timeinterval_ROI, deadtime)

              fullEcounts = edepos
              fullEcounts_err = edepos ** 0.5
              #fulltime_interval= dtcor_GECAM.deadtime_cor(np.sum(fullEcounts[idx]), timeinterval_ROI, deadtime)

              BGEcounts = BGcount_ROI
              BGEcounts_err = BGcounterr_ROI
              #BGtime_interval = dtcor_GECAM.deadtime_cor(np.sum(BGEcounts[idx]), timeinterval_ROI, deadtime)


              '''comment tentatively'''
              '''
              nettime_interval[idx]=dtcor_GECAM.deadtime_cor(timeinterval_ROI, deadtime_idetec, fullcountsSum, netcountsSum)
              fulltime_interval[idx]=dtcor_GECAM.deadtime_cor(timeinterval_ROI, deadtime_idetec, fullcountsSum, fullcountsSum)
              BGtime_interval[idx]=dtcor_GECAM.deadtime_cor(timeinterval_ROI, deadtime_idetec, fullcountsSum, BGcountsSum)
              '''
              '''for MC, use no deadtime tentatively'''
              nettime_interval = timeinterval_ROI
              fulltime_interval =timeinterval_ROI
              BGtime_interval= timeinterval_ROI



          return netEcounts, netEcounts_err, nettime_interval, fullEcounts, \
                 fullEcounts_err, fulltime_interval, BGEcounts, BGEcounts_err, \
                 BGtime_interval
