import os
import sys
sys.path.append(os.path.abspath(os.path.dirname(__file__)+'/'+'..'))
from tools import *
from data import *
from xspec import *
import matplotlib.pyplot as plt
def setspectrum_xspec(specfitsfilename, type_spec, detID_selected,
                      rangeindex_selected,
                      rspFileDirlist):
    spec = [0] * len(detID_selected)
    for i in range(len(detID_selected)):
        idetec = detID_selected[i]
        index = rangeindex_selected[idetec]
        sidetec = str(idetec + 1)
        sidetec = sidetec.zfill(3)
        netfitsname = specfitsfilename[0][i]
        BGfitsname = specfitsfilename[1][i]
        fullfitsname = specfitsfilename[2][i]
        # make rsp filename
        '''
        add rsp filename
        '''
        rspfile=rspFileDirlist[i]
        if type_spec == 'net': #net, need to load rsp file
           spec[i]=Spectrum(netfitsname)
           spec[i].response=rspfile
        elif type_spec =='full': #full, need to load rsp file, backfile
           spec[i]=Spectrum(fullfitsname)
           spec[i].response=rspfile
           spec[i].background=BGfitsname
 
        print('netfitsname:' +netfitsname)
        print('rspfile:' + rspfile)
        print('idetec:' ,idetec)
        print('rangeindex_selected', rangeindex_selected)
        # select the range
        str_ignore = "**-" + str(index[0]) + " " + str(index[1]) + "-**"
        spec[i].ignore(str_ignore)

def setmodel_xpec(type_model, paralist):
    #if use xspec  and GECAM model
    if glv.flag_resetfit!=0:
       AllModels.clear() 
    if type_model==0: 
       m=Model("powerlaw") #in Xspec model       
    elif type_model==1:
       m=Model("grbm") #My model
    elif type_model==2:
       m=Model("bmc") #My model
    elif type_model==3:
       m=Model("bkn2pow")  #My model 
    elif type_model==4:
       m=Model("bbody")  #in Xspec model
    elif type_model==5:  #DIY model
       m=Model(glv.DIYmodelname)

    # set state, initial value, delta, min, bot, top, max
    '''
    for i in range(m.nParameters):
        print('i:'+str(i))
        par_i=m(i+1)
        if paralist[i][0]=='frozen':
           par_i.frozen=True
        else:
           print(paralist[i][1:])
           par_i.values=paralist[i][1:]
    '''

def setfitmethod_xspec(fitmethod):
    if fitmethod==0:
       Fit.statMethod = "chi"
    elif fitmethod==1:   
       Fit.statMethod = "lstat"
    elif fitmethod==2:
       Fit.statMethod = "cstat"
    elif fitmethod==3:
       Fit.statMethod = "pstat"


def setiteration_xspec(niter):
    Fit.nIterations=niter


def setplot(selstr):
    Plot.device = "/xs"
    Plot.yLog = True
    Plot.xAxis = "KeV"
    if len(selstr)==1:
       Plot(selstr[0])
    elif len(selstr)==2:
       Plot(selstr[0], selstr[1])
    elif len(selstr)==3:
       Plot(selstr[0], selstr[1], selstr[2])


def fillfitres(restype, elow, ehigh): #restype= single, batch, best
    fitres_para=[]
    fitres_cov=Fit.covariance
    index=[] 
    for i in range(m.nParameters):
        if i==0:
           j=0
           index.append(0) 
        else:
           j=index[i-1]+i+1 
           index.append(j) 
        fitres_para.append([m(i).values, fitres_cov(j)**0.5])
         
    fitres_ndof=Fit.dof
    fitres_reducedChi2=Fit.testStatistic 
    
    # cal flux fluence
    elimitstr=str(elow) + " " +str(ehigh) + " err"
    AllModels.calcFlux(elimitstr) 
    fluence_flux_tuple=Allmodel.flux(elimitstr)
    flux=[fluence_flux_tuple[3], fluence_flux_tuple[4]-fluence_flux_tuple[3], fluence_flux_tuple[5]-fluence_flux_tuple[3]]
    fluence=[fluence_flux_tuple[0], fluence_flux_tuple[1]-fluence_flux_tuple[0], fluence_flux_tuple[2]-fluence_flux_tuple[0]]

    fitres_Edepos_dict={
        'restype': glv.restype, 
        #'tstart_end_Efit':[tstart, tend],
        'detID_selected':glv.detID_fitEdepos_selected,
        'fitmethod':glv.fitmethod,
        'fitmodel': glv.fitmodel,
        'para':fitres_para,
        'cov':fitres_cov,
        'n.d.o.f': fitres_ndof,
        'reduced chi2':fitres_reducedChi2,
        'flux(photons)':flux,
        'fluence(photons*cm-2*s-1)': fluence
         } 
    
    return fitres_Edepos_dict 


def savefitresults_xpec(savefilename):
    Xset.save(savefilename) 


def def_specfitsfilename(detID_selected, filedir_front, iTR):
    specfitsfilename_net=[0]*len(detID_selected)
    specfitsfilename_BG=[0]*len(detID_selected)
    specfitsfilename_full=[0]*len(detID_selected)
    for i in range(len(detID_selected)):
        idetec=detID_selected[i]
        '''make net, BG, full file fits name'''
        sidetec= str(idetec+1)
        sidetec=sidetec.zfill(3)
        if iTR ==False:
           netfitsname = filedir_front+'netEdepos'+ sidetec+'.fits'
           BGfitsname = filedir_front+'BGEdepos'+ sidetec+'.fits'
           fullfitsname = filedir_front+'fullEdepos'+ sidetec+'.fits'
           specfitsfilename_net[i]= netfitsname
           specfitsfilename_BG[i]= BGfitsname
           specfitsfilename_full[i]= fullfitsname
        else:
            netfitsname = filedir_front + 'netEdepos' + sidetec +'_tbin'+iTR + '.fits'
            BGfitsname = filedir_front + 'BGEdepos' + sidetec + '_tbin'+iTR +'.fits'
            fullfitsname = filedir_front + 'fullEdepos' + sidetec + '_tbin'+iTR +'.fits'
            specfitsfilename_net[i] = netfitsname
            specfitsfilename_BG[i] = BGfitsname
            specfitsfilename_full[i] = fullfitsname
    return [specfitsfilename_net, specfitsfilename_BG,specfitsfilename_full]


def fitxspec(rspFileDirlist):
    '''for logfile, if logfile exists,remove it'''
    if glv.flag_resetfit !=0:
       AllData.clear()
       AllModels.clear()
       print('data cleared!')
    logfilename=glv.genfitsDir + 'logfile'
    if os.path.isfile(logfilename):
       os.remove(logfilename)

    Xset.openLog(logfilename)
    Xset.logChatter=10
    type_specfit=glv.type_specfit
    detec_selected=glv.detID_selected
    rangeindex_selected=glv.rangeindex_selected 
    type_model=glv.fitmodel
    paralist=glv.paralist
    fitmethod= glv.fitmethod
    niter=glv.Niter
    plotselection=glv.plotselections
 
    # set arguments
    #setspectrum_xspec(type_specfit,  detec_selected, rangeindex_selected)
    print('detec_selected' +str(detec_selected))
    specfitsfilename=def_specfitsfilename(detec_selected, glv.genfitsDir, False)
    print('specfitsfilename' +str(specfitsfilename))
    setspectrum_xspec(specfitsfilename,type_specfit, detec_selected,
                      rangeindex_selected,
                      rspFileDirlist)
    #setmodel_xpec(type_model,paralist)
    if glv.flag_resetfit!=0:
       AllModels.clear()
    if type_model==0:
       m=Model("powerlaw") #in Xspec model
    elif type_model==1:
       m=Model("grbm") #My model
    elif type_model==2:
       m=Model("funccomptonized") #My model
    elif type_model==3:
       m=Model("funcSBpowerlaw")  #My model
    elif type_model==4:
       m=Model("bbody")  #in Xspec model
    elif type_model==5:  #DIY model
       if glv.DIYmodelstr != 'e.g. Mymodelname':
          m=Model(glv.DIYmodelstr)
       elif glv.combmodelstr!= 'e.g. phabs*po + ga':
          m=Model(glv.combmodelstr)
    setfitmethod_xspec(fitmethod)
    setiteration_xspec(niter)
    
    #perform fit
    Fit.perform()
    

    #Plot
    setplot(plotselection)
    #save fit result 
    #savefitresplot() 

    #update the status
    glv.flag_resetfit=glv.flag_resetfit+1
    Xset.closeLog()

    fitres_para = []
    fitres_cov = Fit.covariance
    npara = m.nParameters
    fitstatus = 'SUC'
    print(int(npara * (npara + 1) / 2))
    print(len(list(fitres_cov)))
    if int(npara * (npara + 1) / 2) > len(list(fitres_cov)):
        fitstatus = 'FAIL'
    index = []
    for i in range(npara):
        if i == 0:
            j = 0
            index.append(0)
        else:
            j = index[i - 1] + i + 1
            index.append(j)
        '''int(npara * (npara - 1)/2) > len(list(fitres_cov)), 
        cov is OK, or failed, the default value for failed is 99999.9 '''
        if fitstatus == 'SUC':
            fitres_para.append([m(i + 1).values[0], fitres_cov[j] ** 0.5])
        else:
            fitres_para.append([m(i + 1).values[0], 99999.9])

    fitres_ndof = Fit.dof
    fitres_reducedChi2 = Fit.testStatistic / fitres_ndof
    print('dof' + str(fitres_ndof))
    print('chi2/dof' + str(fitres_reducedChi2 / fitres_ndof))

    '''cal flux fluence'''
    elow = glv.lEbin_mid[detec_selected[0]][0]
    ehigh = glv.lEbin_mid[detec_selected[0]][-1]
    if len(rangeindex_selected[detec_selected[0]]) == 2:
        elow = glv.lEbin_mid[detec_selected[0]][rangeindex_selected[detec_selected[0]][0]]
        ehigh = glv.lEbin_mid[detec_selected[0]][rangeindex_selected[detec_selected[0]][1]]
    elimitstr = str(elow) + " " + str(ehigh) + " 3 err"
    fluence_flux_tuple = [0, 0, 0, 0, 0, 0]
    if fitstatus == 'FAIL':
        pass
    else:
        AllModels.calcFlux(elimitstr)
        specData = AllData(1)
        fluence_flux_tuple = specData.flux

    flux = [fluence_flux_tuple[3], fluence_flux_tuple[4] - fluence_flux_tuple[3],
            fluence_flux_tuple[5] - fluence_flux_tuple[3]]
    fluence = [fluence_flux_tuple[0], fluence_flux_tuple[1] - fluence_flux_tuple[0],
               fluence_flux_tuple[2] - fluence_flux_tuple[0]]

    fitres_Edepos_dict = {
        'detID_selected': detec_selected,
        'fitmethod': glv.fitmethod,
        'fitmodel': glv.fitmodel,
        'fitstatus': fitstatus,
        'para': fitres_para,
        'cov': fitres_cov,
        'n.d.o.f': fitres_ndof,
        'reduced chi2': fitres_reducedChi2,
        'flux(photons)': flux,
        'fluence(ergs/cm^2/s)': fluence
    }
    return fitres_Edepos_dict


